#include "Ball.h"


Ball::Ball (float x, float y, float z, float vx, float vy, float vz, float m, float r, float dt) {
	points      = new Point[1];
	state       = new float[6];
	forceAccums = new float[3];
	masses      = new float[1];
	fixed       = new bool[1];

	// init the point
	points[0].position   = &state[0];
	points[0].velocity   = &state[3];
	points[0].forceAccum = &forceAccums[0];
	points[0].mass       = &masses[0];
	points[0].fixed      = &fixed[0];

	// init the fields
	state[0 + DIM_X] = x;
	state[0 + DIM_Y] = y;
	state[0 + DIM_Z] = z;
	state[3 + DIM_X] = vx;
	state[3 + DIM_Y] = vy;
	state[3 + DIM_Z] = vz;
	masses[0]        = m;
	radius           = r;
	fixed[0]         = false;
	tempMass    = 0.0f;

	setForceAccumulatorsWithAcceleration(GX, GY, GZ);
	//applyForces();


//	solver = (Solver *) new Euler(this, dt);
	solver = (Solver*) new LeapFrog(this, dt);
//	solver = (Solver*) new Beeman(this, dt);
}

void Ball::reset(float x, float y, float z, float vx, float vy, float vz) {
	state[0 + DIM_X] = x;
	state[0 + DIM_Y] = y;
	state[0 + DIM_Z] = z;
	state[3 + DIM_X] = vx;
	state[3 + DIM_Y] = vy;
	state[3 + DIM_Z] = vz;

	setForceAccumulatorsWithAcceleration(GX, GY, GZ);
	tempMass    = 0.0f;
	translation->translation.setValue(state);
	translation->touch();

}


int Ball::numPoints() {
	return 1;
}

float Ball::getRadius() {
	return radius;
}

SoNode* Ball::getNode() {
	SoSeparator *root = new SoSeparator;
	root->ref();

	translation = new SoTranslation();
	root->addChild(translation);
	translation->translation.setValue(state);

	color = new SoBaseColor();
	root->addChild(color);
	color->rgb.setValue(0.3f, 0.3f, 1.0f);

	complexity = new SoComplexity();
	root->addChild(complexity);
	complexity->value = 10;

	SoSphere *sphere = new SoSphere();
	root->addChild(sphere);
	sphere->radius = this->radius;	

	root->unrefNoDelete();
	return root;
}

void Ball::calculateForces() {

	// 1. calculate the involved forces
	setForceAccumulatorsWithAcceleration(GX, GY, GZ);
	//applyForces();
}

void Ball::step() {
	// 2. perform a step
	solver->performStep();

	// 3. tell coin that the position has changed
	translation->translation.setValue(state);
	translation->touch();

//	printf("ball position: %f, %f, %f\n", state[0], state[1], state[2]);
//	printf("ball speed:    %f, %f, %f\n", state[3], state[4], state[5]);
//	printf("ball force:    %f, %f, %f\n\n", forceAccums[0], forceAccums[1], forceAccums[2]);
}

//deferred feature
Ball::Ball() {}



