#include "CollisionDetector.h"

CollisionDetector::CollisionDetector() {

}

void CollisionDetector::addBall(Ball *ball) {
	balls.append(*ball);
}

void CollisionDetector::addMembran(ElasticMembran *membran) {
	membrans.append(*membran);
}

void CollisionDetector::callback() {

	int i, j;
	for (i=0; i < balls.getLength(); i++) {
		for (j=0; j < membrans.getLength(); j++) {
			collideBallWithMembran(balls[i], membrans[j].points, membrans[j].numPoints());
		}
		/*
		for (j=0; j < i; j++) {
			collideBallWithBall(balls[i], balls[j])
		}
		*/
	}
}

void CollisionDetector::calculateForces() {
	int i, j;
	for (i=0; i < balls.getLength(); i++) {
		// Restore original mass of ball, i.e. without the masses of the collision one
		// time step before:
		balls[i].masses[0] -= balls[i].tempMass;
		for (j=0; j < membrans.getLength(); j++) {
			applyForce(balls[i], membrans[j].points, membrans[j].numPoints());
		}
		// Add the mass of the points to the ball, s.t. we have one big object of 
		// colliding with the ball and the points:
		balls[i].masses[0] += balls[i].tempMass;
	}
}


/*
** Interpret colliding Points and Balls as one big object: one common mass and 
** one common force.
*/
void CollisionDetector::applyForce(Ball ball, Point *points, int numPoints) {
	int i;
	for (i = 0; i < numPoints; i++) {
		if (points[i].didCollide) {
			ball.tempMass += points[i].mass[0];
			ball.forceAccums[0] += points[i].forceAccum[0];
			ball.forceAccums[1] += points[i].forceAccum[1];
			ball.forceAccums[2] += points[i].forceAccum[2];
		}
	}

	for (i = 0; i < numPoints; i++) {
		if (points[i].didCollide) {
//			points[i].forceAccum[0] += ball.forceAccums[0];
//			points[i].forceAccum[1] += ball.forceAccums[1];
//			points[i].forceAccum[2] += ball.forceAccums[2];

//			points[i].forceAccum[0] = ball.forceAccums[0];
//			points[i].forceAccum[1] = ball.forceAccums[1];
//			points[i].forceAccum[2] = ball.forceAccums[2];

			points[i].velocity[0] = ball.state[3];
			points[i].velocity[1] = ball.state[4];
			points[i].velocity[2] = ball.state[5];
		}
	}
} 


void CollisionDetector::collideBallWithMembran(Ball ball, Point *points, int numPoints) {
	float radius = ball.getRadius() + OFFSET;
	SbVec3f massPoint, ballCenter, vec;
	ballCenter.setValue(ball.state);
	for (int i=0; i < numPoints; i++) {
		massPoint.setValue(points[i].position);
		// vector from ball center to mass point
		vec = massPoint - ballCenter;
		if (vec.length() < radius) {
			// collide point with Ball
			handleBallPointCollision(ball, points[i], vec, radius);
			// remember, that point collided one time step before allready:
			if(!points[i].fixed) {
				points[i].didCollide = true;
			} else {
				points[i].didCollide = false;
			}
		} else {
			points[i].didCollide = false;
		}
	}
}


void CollisionDetector::handleBallPointCollision(Ball ball, Point point, SbVec3f vec, float radius) {
	float d, j;
	Point ballPoint = ball.points[0];

	d = vec.normalize();
	d = radius - d;

	// 1. move point out of the ball
	if (!point.fixed[0]) {
		point.position[0] += vec[0] * d;
		point.position[1] += vec[1] * d;
		point.position[2] += vec[2] * d;
	} else {
		// 2. move point out of the ball
		ballPoint.position[0] -= vec[0] * d;
		ballPoint.position[1] -= vec[1] * d;
		ballPoint.position[2] -= vec[2] * d;
	}

	// if this point did collide before we are done, because we look at the ball and 
	// its colliding points as one big object with one mass and one force, which
	// were calculated before in calculateForces():
	// 
	if (point.didCollide) {
		return;
	}

	// 2. calculate the impulse (total inelastic)

	//       m1 * m2
	//  j = ---------- * (1 + E) * <( v1 - v2 ), n>
	//       m1 + m2

	j = (1+E) * (ballPoint.mass[0]*point.mass[0]) / (ballPoint.mass[0]+point.mass[0]);
	j *= ((ballPoint.velocity[0] - point.velocity[0]) * vec[0]) +
		    ((ballPoint.velocity[1] - point.velocity[1]) * vec[1]) +
		    ((ballPoint.velocity[2] - point.velocity[2]) * vec[2]);

	if (!point.fixed[0]) {
		// 3. calculate change of velocity for both ball and point
		d = -j/ballPoint.mass[0];
		ballPoint.velocity[0] += d * vec[0];
		ballPoint.velocity[1] += d * vec[1];
		ballPoint.velocity[2] += d * vec[2];

		d = j/point.mass[0];
		point.velocity[0] += d * vec[0];
		point.velocity[1] += d * vec[1];
		point.velocity[2] += d * vec[2];
	} else {
		// 3. calculate change of velocity for the ball
		d = -2*j/ballPoint.mass[0];
		ballPoint.velocity[0] += d * vec[0];
		ballPoint.velocity[1] += d * vec[1];
		ballPoint.velocity[2] += d * vec[2];
	}
}

