#include "ElasticMembran.h"


/**
 * Cunstructor
 * creates an elastic membrane
 *
 * TODO:
 * make more generic constructos once the project runs
 */
ElasticMembran::ElasticMembran(int n_, int m_, float dx, float dy, float mass, float dt) {
	n = n_;
	m = m_;

	// create arrays
	points      = new Point[n*m];
	state       = new float[n*m*6];
	forceAccums = new float[n*m*3];
	masses      = new float[n*m];
	fixed       = new bool[n*m];

	Spring *s; 

	// inititalize the arrays
	for (int i = 0; i < n; i++)
	{
		for (int j = 0; j < m; j++)
		{	// set up the points in the points-array:
			points[i*m + j].position   = &state[i*m*3 + j*3];
			points[i*m + j].velocity   = &state[n*m*3 + i*m*3 + j*3];
			points[i*m + j].forceAccum = &forceAccums[i*m*3 + j*3];
			points[i*m + j].mass       = &masses[i*m + j];
			points[i*m + j].fixed      = &fixed[i*m + j];
			points[i*m + j].didCollide = false;

			// set up the arrays for the solver:
			state[i*m*3 +  j*3 + DIM_X]         = i*dx - (n*dx)/2;
			state[i*m*3 +  j*3 + DIM_Y]         = j*dy - (m*dy)/2;
			state[i*m*3 +  j*3 + DIM_Z]         = 0.0;
			state[n*m*3 + i*m*3 +  j*3 + DIM_X] = 0.0;
			state[n*m*3 + i*m*3 +  j*3 + DIM_Y] = 0.0;
			state[n*m*3 + i*m*3 +  j*3 + DIM_Z] = 0.0;
			masses[i*m + j]                     = mass;
		}
	}

	// Setting up the constraints (fixing points) and forces:
	float diagLength = sqrt(dx*dx + dy*dy) * LENGTH;
	for (int i = 0; i < n; i++)
	{
        for (int j = 0; j < m; j++)
		{
            if ((i == 0) | (i == n-1) | (j == 0) | (j == m-1)) {
				fixed[i*m + j] = true;
			} else {
				fixed[i*m+j] = false;
			}
			
			// point not in the top row
            if (i != 0) {
				// spring to the point above
				s = new Spring(&points[i*m + j], &points[(i-1)*m + j], STIFFNESS, DAMPING, dy*LENGTH);
				forces.append(*s);

				// point not in the first column
				if (j != 0) {
					// spring to the diagonal left above point
					s = new Spring(&points[i*m + j], &points[(i-1)*m + j-1], STIFFNESS, DAMPING, diagLength);
					forces.append(*s);
				}

				// point not in the last column
				if (j != m-1) {
					// spring to the diagonal right above point
					s = new Spring(&points[i*m + j], &points[(i-1)*m + j+1], STIFFNESS, DAMPING, diagLength);
					forces.append(*s);
				}
			}

			// point not in the first column
            if (j != 0) {
				// spring to the left point
				s = new Spring(&points[i*m + j], &points[i*m + j-1], STIFFNESS, DAMPING, dx*LENGTH);
				forces.append(*s);
			}
		}
	}

	// init the forces
	setForceAccumulatorsWithAcceleration(GX, GY, GZ);
	applyForces();


//	solver = (Solver *) new Euler(this, dt);
	solver = (Solver*) new LeapFrog(this, dt);
//	solver = (Solver*) new Beeman(this, dt);
}


void ElasticMembran::reset(float dx, float dy) {
	// inititalize the arrays
	for (int i = 0; i < n; i++)
	{
		for (int j = 0; j < m; j++) 
		{
			// set up the arrays for the solver:
			state[i*m*3 +  j*3 + DIM_X]         = i*dx - (n*dx)/2;
			state[i*m*3 +  j*3 + DIM_Y]         = j*dy - (m*dy)/2;
			state[i*m*3 +  j*3 + DIM_Z]         = 0.0;
			state[n*m*3 + i*m*3 +  j*3 + DIM_X] = 0.0;
			state[n*m*3 + i*m*3 +  j*3 + DIM_Y] = 0.0;
			state[n*m*3 + i*m*3 +  j*3 + DIM_Z] = 0.0;

			points[i*m + j].didCollide = false;
		}
	}
	// init the forces
	setForceAccumulatorsWithAcceleration(GX, GY, GZ);
	applyForces();
	coinCoords->point.touch();
}



/**
 * Returns a SoNode to be added to a Coin3D scene graph
 *
 */
SoNode* ElasticMembran::getNode() {
	SoSeparator *root = new SoSeparator();
	root->ref();
	coinCoords = new SoCoordinate3();
	root->addChild(coinCoords);

	SoBaseColor *color = new SoBaseColor();
	root->addChild(color);
	color->rgb.setValue(1.0, 0.0, 0.0);

	coinCoords->point.setValuesPointer(numPoints(), state);

	SoQuadMesh * mesh = new SoQuadMesh;
 	SoShapeHints *shapeHint = new SoShapeHints;
	shapeHint->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE;
	root->addChild(shapeHint);
	root->addChild(mesh);

    mesh->verticesPerRow = m;
    mesh->verticesPerColumn = n;

	root->unrefNoDelete();
	return root;
};


/**
 * This Function is called by the SoTimeSensor
 * It performes one step of the ode_solver
 */
void ElasticMembran::calculateForces() {

	// 1. calculate the involved forces
	setForceAccumulatorsWithAcceleration(GX, GY, GZ);
	applyForces();
 }

 void ElasticMembran::step() {
	// 2. perform a step
	solver->performStep();

	// 3. tell coin thath the positions have changed
	coinCoords->point.touch();

//	int i = (numPoints()/2)+50;
//	printf("position: %f, %f, %f\n", state[i + 0], state[i + 1], state[i + 2]);
//	printf("speed:    %f, %f, %f\n", state[(numPoints()*3) + i + 0], state[(numPoints()*3) + i + 1], state[(numPoints()*3) + i + 2]);
//	printf("force:    %f, %f, %f\n\n", forceAccums[i + 0], forceAccums[i + 1], forceAccums[i + 2]);
}


/**
 * The number of points
 */
int ElasticMembran::numPoints() {
	return n*m;
}


// deferred feature
ElasticMembran::ElasticMembran() {}

