#ifndef ELASTIC_MEMBRANE_H
#define ELASTIC_MEMBRANE_H

	#include <Inventor/nodes/SoNode.h>
	#include <Inventor/nodes/SoSeparator.h>
	#include <Inventor/nodes/SoCoordinate3.h>
	#include <Inventor/nodes/SoQuadMesh.h>
	#include <Inventor/nodes/SoShapeHints.h>
	#include <Inventor/nodes/SoBaseColor.h>

	#include <math.h>

	#include "MassPoint.h"

	#define STIFFNESS	20.0f	// stiffnes of the spring
	#define LENGTH		0.01f	// factor by witch the spring is shorter as the initial distance between two points
	#define DAMPING		0.005f	// damping applied to each spring


	class ElasticMembran: public MassPoint {
		public:
			/**
			 * Konstruktor
			 */
			ElasticMembran(int n, int m, float dx, float dy, float mass, float dt);
			ElasticMembran();

			/** 
			 * Gibt eine Node zurck, die in einen Coin3D Scene Graph gehngt werden kann.
			 * An dieser Node hngen: - SoCoordinate3D mit Field SoMFVec3f, welches mittels
			 *                          setPointerValues(float *positions) gesetzt ist.
			 *                        - renderfhige Nodes, welche die Membran darstellen (z.Bsp
			 *                          TriangleSets)
			 * Der Benutzer dieser Klasse muss lediglich diese Node in den Graph setzen.
			 */
			SoNode* getNode();

			/**
			 * This function must be be called in order to
			 * do a timestep of the system
			 */
			virtual void calculateForces();
			virtual void step();

			virtual int numPoints();

			void reset(float dx, float dy);

		private:
			/**
			 * The CoordinateNode in the scene graph
			 * neede to tell the application if data changed
			 */
			SoCoordinate3 *coinCoords;

			int n;
			int m;



	};

#endif