#include "Force.h"
#include <stdio.h>


Spring::Spring(Point *p1, Point *p2, float stiff, float damp, float length) {
	point1 = p1;
	point2 = p2; 
	stiffness = stiff;
	damping = damp; 
	initialLength = length;
}

/**
 * Adds the spring force to the force accumulators of the two points.
 * Spring force = stiffness*|p1 - p2| - damping*|p1.v - p2.v|
 */
void Spring::applyForce() {
	float length;
	SbVec3f spring, restoreForce, v1, v2;

	v1.setValue(point1->position);
	v2.setValue(point2->position);
	spring = v2 - v1;

	length = spring.normalize();
	restoreForce = (stiffness * (length - initialLength)) * spring;

//	printf("point1->position = %f, %f, %f\n", point1->position[0], point1->position[1], point1->position[2]);
//	printf("point2->position = %f, %f, %f\n", point2->position[0], point2->position[1], point2->position[2]);
//	printf("springforce = %f, %f, %f\n", v1[0], v1[1], v1[2]);

	if (!*(point1->fixed)) {
		point1->forceAccum[0] +=  restoreForce[0] - damping * point1->velocity[0];
        point1->forceAccum[1] +=  restoreForce[1] - damping * point1->velocity[1];
        point1->forceAccum[2] +=  restoreForce[2] - damping * point1->velocity[2];
	}
	if ((!*point2->fixed)) {
        point2->forceAccum[0] += -restoreForce[0] - damping * point2->velocity[0];
        point2->forceAccum[1] += -restoreForce[1] - damping * point2->velocity[1];
        point2->forceAccum[2] += -restoreForce[2] - damping * point2->velocity[2];
	}
	
	/*
	*	TODO: Bringts das? Anstelle der halben Kraft addieren wre es doch sinnvoller, 
	*		  stiffness*(400*initialLength - initialLength)*spring zu addieren, was
	*		  einer konstanten MaximalKraft entspricht...
	*	TODO: Soll Damping relativ zur Gesamtgeschwindigkeit (wie unten gemacht) oder
	*		  relativ zur Geschwindigkeit parallel zur Feder gemacht werden?
	*/ 
	length = length - 400*initialLength;
	if (length > 0) {
		if (!*(point1->fixed)) {
			point1->forceAccum[0] += 0.5f * (restoreForce[0] - damping * point1->velocity[0]);
			point1->forceAccum[1] += 0.5f * (restoreForce[1] - damping * point1->velocity[1]);
			point1->forceAccum[2] += 0.5f * (restoreForce[2] - damping * point1->velocity[2]);
		}
		if ((!*point2->fixed)) {
			point2->forceAccum[0] += 0.5f *(-restoreForce[0] - damping * point2->velocity[0]);
			point2->forceAccum[1] += 0.5f *(-restoreForce[1] - damping * point2->velocity[1]);
			point2->forceAccum[2] += 0.5f *(-restoreForce[2] - damping * point2->velocity[2]);
		}
	}

//	printf("point1->forceaccum  = %f, %f, %f\n", point1->forceAccum[0], point1->forceAccum[1], point1->forceAccum[2]);
//	printf("point2->forceaccum  = %f, %f, %f\n", point2->forceAccum[0], point2->forceAccum[1], point2->forceAccum[2]);
}

void Force::applyForce() {}
Spring::Spring() {}



