#ifndef MASS_POINT_H
#define MASS_POINT_H

	#include <Inventor/lists/SbList.h>
	#include "ODE_System.h"
	#include "Force.h"
	#include "Solver.h"


	#define DT       0.01f		// the length of one time step

	// the gravity
	#define GX		0.0f
	#define GY		0.0f
	#define GZ		9.0f

	// offset for the coordinates
	#define DIM_X	0
	#define DIM_Y	1
	#define DIM_Z	2



	class MassPoint: public ODE_System {
		public:
			/**
			 * Calculates the derivatives of the state
			 */
			virtual void calculateDerivatives (float derivatives[]);

			/**
			 * Calculate only the accelerations of the state:
			 */
			virtual void calculateAccelerations(float accelerations[]);

			/**
			 * dimension of the array 'state'
			 */
			int getDimension();


			/**
			 * the number of points in this system
			 */
			virtual int numPoints();


			/**
			 * This functions must be be called in order to
			 * do a timestep of the system
			 */
			virtual void calculateForces();
			virtual void step();


			/**
			 * The ode_solver
			 */
			Solver *solver;


			/**
			 * The fields and functions handling the forces
			 */
			// TODO: wenn man eine liste von 'Force' macht wird
			//		 Force::applyForce() ausgefhrt und nicht wie erwartet Spring::applyForce()
			SbList<Spring> forces;
			void setForceAccumulatorsWithForce(float fx, float fy, float fz);
			void setForceAccumulatorsWithAcceleration(float gx, float gy, float gz);
			void applyForces();


			/**
			 * array of pointer to all the point objects in the system:
			 */
			Point *points;

			/**
			 * array containing the accumulated forces attacking each points
			 */
			float  *forceAccums;

			/**
			 * array containing the mass of all points
			 */
			bool   *fixed;

			/**
			 * array of boolean
			 * true menas point may not be moved
			 */
			float  *masses;

		private:
	};

#endif