#ifndef SOLVER_H
#define SOLVER_H

	#include "ODE_System.h"

	class Solver {
		public:
			/**
			 * Performs a step of the Solver
			 */
			virtual void performStep();

			/**
			 * The stepsize of the solver
			 */
			float dt;

			bool position;

			/**
			 * The ODE_System this solver solves
			 */
			ODE_System *odeSystem;

			/**
			 * Some usefull array operations
			 */
			void scaleVector(float vector[], float factor, int length);
			void addVector(float vector[], float summand[], int length);
			void addScaledVector(float vector[], float summand[], float factor, int length);

		private:
	};



	class Euler: public Solver {
		public:
			Euler(ODE_System *system_, float dt_);
			virtual void performStep();

		private:
			float *derivatives;
	};



	class LeapFrog: public Solver {
		public:
			LeapFrog(ODE_System *system_, float dt_);
			virtual void performStep();

		private:
			float *derivatives;
	};



	class Beeman: public Solver {
		public:
			Beeman (ODE_System *system_, float dt_);
			virtual void performStep();

		private:
			float *derivatives;
			// array storing a(t-h):
			float *a0;
			// array storing a(t):
			float *a1;

			// Boolean flag to remember in which state we are, either updating the positions
			// or the velocities:
//			bool position;
	};

#endif