#include "application.h"





int main(int args, char **argv) {
	HWND window = SoWin::init(argv[0]);
	if (window == NULL) exit(1);

	SoWinExaminerViewer *viewer = new SoWinExaminerViewer(window);
	viewer->setViewing(false);

	// Framecounter:
	coin_setenv("COIN_SHOW_FPS_COUNTER", "1", TRUE);

	SoSeparator *root = new SoSeparator;
	root->ref();

	SoEventCallback *eventcb = new SoEventCallback();
	eventcb->addEventCallback(SoKeyboardEvent::getClassTypeId(), keyboardEventCallback, em);
	root->addChild(eventcb);

	// add camera and lights
	SoPerspectiveCamera *camera = new SoPerspectiveCamera;
	root->addChild(camera);
	camera->position.setValue(10.0f, 10.0f, 10.0f);
	camera->pointAt(SbVec3f().setValue(0.0f,0.0f,0.0f), SbVec3f().setValue(-1.0f,-1.0f,2.0f));

	SoDirectionalLight *light1 = new SoDirectionalLight;
	root->addChild(light1);
	light1->intensity = 0.1f;
	light1->direction.setValue(0.0f, 0.0f, 1.0f);

	SoSpotLight *light = new SoSpotLight;
	root->addChild(light);
	light->location.setValue(-10.0f, 15.0f, 20.0f);
	light->direction.setValue(10.0f, -15.0f, 20.0f);
	light->intensity = 1.0f;
	light->dropOffRate = 0.9f;
	light->cutOffAngle = 0.9f;



	em = new ElasticMembran(N, M, DX, DY, MASS, DT);
	root->addChild(em->getNode());

	ball = new Ball(0.0f, 0.0f, 5.0f, 0.0f, 0.0f, -20.0f, BALL_MASS, RADIUS, DT);
	root->addChild(ball->getNode());

	detector = new CollisionDetector();
	detector->addBall(ball);
	detector->addMembran(em);

	// create a new timer sensor and set time in milliseconds:
	timeSensor = new SoTimerSensor;
    timeSensor->setFunction(callback);
    timeSensor->setBaseTime(SbTime::getTimeOfDay());
    timeSensor->setInterval(INTERVAL);

	viewer->setSceneGraph(root);
	viewer->show();

	SoWin::show(window);
	SoWin::mainLoop();

	root->unref();
	delete viewer;

	return 0;
}

void callback(void *data, SoSensor *s) {
	em->calculateForces();
	ball->calculateForces();
	// ball:   union the forces and masses of itself and its colliding points (of the previous step):
	// points: set one common velocity (the one of the ball) to these objects:
	detector->calculateForces();
	em->step();
	ball->step();
	// - Move colliding points out of ball
	// - Determine new colliding points and calculate their new velocity (by means of the
	//   impulse conservation...)
	// - Determine still colliding points and points no longer colliding
	detector->callback();
}

void keyboardEventCallback(void *data, SoEventCallback *node) {
	const SoEvent *event = node->getEvent();

	if (SO_KEY_PRESS_EVENT(event, R)) {
		ball->reset(0.0f, 0.0f, 5.0f, 0.0f, 0.0f, -20.0f);
		em->reset(DX, DY);
		node->setHandled();
	} else if (SO_KEY_PRESS_EVENT(event, P)) {
		if (timeSensor->isScheduled()) {
			timeSensor->unschedule();
		} else {
			timeSensor->schedule();
		}
		node->setHandled();
	}
}


